import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/beans/video_tik_entry.dart';
import 'package:eyflutter_tik_video/beans/video_tik_tab.dart';
import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_loading_model.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikPlayer {
  final VideoTikEntry tikEntry;

  final FijkPlayer player;

  final VideoPlayProgressModel progressModel;

  final VideoPlayerIconModel playerIconModel;

  final VideoLoadingModel loadingModel;

  final CollectModel collectModel;

  final ShareModel shareModel;

  TikPlayer(
      {required this.tikEntry,
      required this.player,
      required this.progressModel,
      required this.playerIconModel,
      required this.loadingModel,
      required this.collectModel,
      required this.shareModel});
}

class VideoTikListModel extends CusChangeNotifier {
  PageController pageController = PageController();

  //当前索引
  ValueNotifier<int> index = ValueNotifier<int>(0);

  //播放列表
  List<TikPlayer> items = [];

  //是否前端显示页面
  bool isFrontPage = false;

  VideoTikTab? tikTab;

  Function(VideoTikTab tikTab)? nextPage;

  //数据是否加载中
  bool dataLoading = false;

  //获取播放器
  FijkPlayer? player(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.player;
  }

  //获取数据项
  VideoTikEntry? entry(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.tikEntry;
  }

  //播放进度
  VideoPlayProgressModel? progress(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.progressModel;
  }

  VideoPlayerIconModel? playerIcon(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.playerIconModel;
  }

  VideoLoadingModel? loadingModel(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.loadingModel;
  }

  CollectModel? collect(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.collectModel;
  }

  ShareModel? share(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.shareModel;
  }

  //获取当前播放器
  FijkPlayer? get currentPlayer => player(index.value);

  //播放视频数
  int get count => items.length;

  //视频翻页
  void videoTurn() {
    var p = pageController.page ?? 0;
    if (p % 1 == 0) {
      int target = p ~/ 1;
      if (index.value == target) {
        return;
      }
      var oldIndex = index.value;
      var newIndex = target;
      TikUtils.instance.recoveryPause(player(oldIndex));
      TikUtils.instance.start(player(newIndex));
      index.value = target;
      //剩余页数小于3时触发加载下一页回调
      var diffCount = count - index.value;
      if (diffCount <= 3 && !dataLoading) {
        dataLoading = true;
        nextPage?.call(tikTab ?? VideoTikTab());
      }
    }
  }

  /// 完成数据装载
  void complete() {
    dataLoading = false;
  }

  //添加视频
  appendVideos(List<VideoTikEntry> list) {
    for (var entry in list) {
      FijkPlayer player = FijkPlayer()
        ..setDataSource(
          entry.videoUrl,
          showCover: true,
          autoPlay: entry.autoPlay,
        );
      items.add(loadPlayer(player, entry, list.indexOf(entry)));
    }
  }

  TikPlayer loadPlayer(FijkPlayer player, VideoTikEntry entry, int position) {
    player.addListener(() => playerListener(position, player, entry));
    VideoPlayProgressModel progressModel = VideoPlayProgressModel();
    VideoPlayerIconModel playerIconModel = VideoPlayerIconModel();
    VideoLoadingModel loadingModel = VideoLoadingModel();
    CollectModel collectModel = CollectModel();
    ShareModel shareModel = ShareModel();
    player.onCurrentPosUpdate.listen(
        (event) => playProgressListen(player.value.duration, event, progressModel, playerIconModel, loadingModel));
    player.onBufferPosUpdate
        .listen((event) => bufferPosUpdate(player.value.duration, event, progressModel, playerIconModel, loadingModel));
    player.onBufferStateUpdate.listen((event) => bufferStateUpdate(event, loadingModel));
    player.onBufferPercentUpdate.listen((percent) => bufferPercentUpdate(percent, player, entry, progressModel));
    var tikPlayer = TikPlayer(
        tikEntry: entry,
        player: player,
        progressModel: progressModel,
        playerIconModel: playerIconModel,
        loadingModel: loadingModel,
        collectModel: collectModel,
        shareModel: shareModel);
    return tikPlayer;
  }

  void bufferPercentUpdate(int percent, FijkPlayer player, VideoTikEntry entry, VideoPlayProgressModel progressModel) {
    if (!progressModel.isFirstStarted && entry.autoPlay && player.state != FijkState.started) {
      TikUtils.instance.start(player);
      progressModel.isFirstStarted = true;
    }
  }

  void bufferStateUpdate(bool state, VideoLoadingModel loadingModel) {
    loadingModel.isLoading = state;
    loadingModel.notifierLazy();
  }

  void bufferPosUpdate(Duration total, Duration duration, VideoPlayProgressModel progressModel,
      VideoPlayerIconModel playerIconModel, VideoLoadingModel loadingModel) {}

  void playProgressListen(Duration total, Duration duration, VideoPlayProgressModel progressModel,
      VideoPlayerIconModel playerIconModel, VideoLoadingModel loadingModel) {
    //播放进度
    var currentTime = duration.inMilliseconds.toDouble();
    var totalTime = total.inMilliseconds.toDouble();
    var progress = currentTime / totalTime;
    progressModel.notifyProgress(progress);
    progressModel.notifierLazy();
    //播放状态
    if (playerIconModel.isPlaying == false) {
      playerIconModel.isPlaying = true;
      playerIconModel.notifierLazy();
    }
    //加载状态
    if (loadingModel.isLoading == true) {
      loadingModel.isLoading = false;
      loadingModel.notifierLazy();
    }
  }

  void playerListener(int pos, FijkPlayer player, VideoTikEntry entry) {
    var playerIconModel = playerIcon(pos);
    if (isFrontPage) {
      if (index.value == pos) {
        if (player.state == FijkState.completed) {
          player.start();
        }
      } else {
        TikUtils.instance.pause(player);
        playerIconModel?.isPlaying = false;
      }
    } else {
      TikUtils.instance.pause(player);
      playerIconModel?.isPlaying = false;
    }
    playerIconModel?.notifierLazy();
  }

  //根据页面切换改变当前视频播放状态
  void changeCurrentPlayState() {
    if (isFrontPage) {
      TikUtils.instance.start(currentPlayer);
    } else {
      TikUtils.instance.pause(currentPlayer);
    }
  }

  //回收视频资源
  void release() {
    for (var item in items) {
      item.player.release();
    }
  }
}
