import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_tik_list_model.dart';
import 'package:eyflutter_tik_video/tiklist/tik_video_view.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class VideoTikListView extends StatefulWidget {
  /// 构建回调
  final Function(VideoTikListModel model) buildCall;

  /// 视频填充方式
  final FijkFit fit;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  final Function(PortraitModel model)? portraitCall;

  final Function(CollectModel model)? collectCall;

  final Function(ShareModel model)? shareCall;

  const VideoTikListView(
      {Key? key,
      required this.buildCall,
      required this.fit,
      required this.progressColor,
      required this.progressBackgroundColor,
      this.portraitCall,
      this.collectCall,
      this.shareCall})
      : super(key: key);

  @override
  _VideoTikListViewState createState() => _VideoTikListViewState();
}

class _VideoTikListViewState extends LifecycleState<VideoTikListView> {
  VideoTikListModel listModel = VideoTikListModel();

  @override
  String get viewName => "videoTikList";

  @override
  void initState() {
    super.initState();
    widget.buildCall.call(listModel);
    listModel.pageController.addListener(listModel.videoTurn);
  }

  @override
  void onPause(String routeName, Map<String, dynamic> arguments) {
    listModel.isFrontPage = false;
    TikUtils.instance.pause(listModel.currentPlayer);
  }

  @override
  void onResume(String routeName, Map<String, dynamic> arguments) {
    listModel.isFrontPage = true;
    TikUtils.instance.start(listModel.currentPlayer);
  }

  @override
  void onDestroy(String routeName, Map<String, dynamic> arguments) {
    listModel.release();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return StateUtils.instance.subscribeNotifierWidget<VideoTikListModel>(listModel, (context, model) {
      return PageView.builder(
        controller: model.pageController,
        pageSnapping: true,
        physics: const ClampingScrollPhysics(),
        scrollDirection: Axis.vertical,
        itemCount: model.count,
        itemBuilder: (context, i) {
          return buildPageItem(context, i, model);
        },
      );
    });
  }

  Widget buildPageItem(BuildContext context, int i, VideoTikListModel model) {
    var player = model.player(i);
    var entry = model.entry(i);
    var progress = model.progress(i);
    var playerIcon = model.playerIcon(i);
    var videoLoadingModel = model.loadingModel(i);
    var collect = model.collect(i);
    var shareModel = model.share(i);
    return TikVideoView(
      key: Key("${entry?.videoUrl ?? ""}$i"),
      singleTapCall: () {
        var playerIconModel = model.playerIcon(model.index.value);
        if (player.state == FijkState.started) {
          TikUtils.instance.pause(player);
          playerIconModel?.isPlaying = false;
        } else {
          TikUtils.instance.start(player);
          playerIconModel?.isPlaying = true;
        }
        playerIconModel?.notifierLazy();
      },
      favoriteCall: () {},
      player: player!,
      buildCall: () {},
      progressModel: progress!,
      playerIconModel: playerIcon!,
      loadingModel: videoLoadingModel!,
      fit: widget.fit,
      progressColor: widget.progressColor,
      progressBackgroundColor: widget.progressBackgroundColor,
      collectModel: collect!,
      shareModel: shareModel!,
      portraitCall: widget.portraitCall,
      collectCall: widget.collectCall,
      shareCall: widget.shareCall,
    );
  }
}
